/*
 SdRecorder is intended to be a sound activated MP3 recorder for square dance
 It is cross-platform for Windows and Linux, ising the Qt multimedia interface
 (QAudioDeviceInfo, QAudioFormat, QAudioInput) to open the sound card and read audio date.
 It uses LAME to convert the raw PCM audio to MP3 (lame_enc.dll on Windows, libmp3lame.so on Linux)
 I want to implement a sound recording level meter (VU meter), sound activated recording,
 automatic track numbering, and ID3V1 tagging. Sound activation optional (manual start/stop).
 I wish to support all sample rates supported by the sound card, both mono and stereo,
 but only 16 bit signed PCM data (all sound cards use this format).
 Settings will be saved in an INI file on both Windows and Linux.
 Settings include the sound card selection, mono/stereo, sample rate, MP3 bitrate,
 sound activation level, and on/off times for triggering the recording.
 I could, in theory, buffer the sound data before recording starts so some (limited)
 pre-trigger sound data could be included in the output file. Nice, but not essential.
 All frequencies, bitrates and channel modes supported by lame will be available.
 I will also support recording stereo input to mono output (L+R)/2,
 and mono input to stereo output (duplicate the input to both output channels).

 I initialize the MP3 stream and open the MP3 file when the recording session is started.
 That way the file is ready for encoding to start as soom as the sound triggers the recording.
 When the recording stops, I will close the MP3 stream and MP3 file, and open the next one
 in preparation for the next recording. In this way the MP3 file is always ready when recording starts.
 */
/*
To create a proper Windows Icon file for the Windows EXE file:
First, create an ICO format bitmap file that contains the icon image.
This can be done with e.g. Microsoft Visual C++:
Select File|New, then select the File tab in the dialog that appears, and choose Icon.
(Note that you do not need to load your application into Visual C++; here we are only using the icon editor.)
Or use GIMP to create the ICO file!
Store the ICO file in your application's source code directory,
for example, with the name myappico.ico.
Then, create a text file called, say, myapp.rc in which you put a single line of text:

    IDI_ICON1               ICON    DISCARDABLE     "myappico.ico"

Finally, assuming you are using qmake to generate your makefiles, add this line to your myapp.pro file:

    RC_FILE = myapp.rc

Regenerate your makefile and your application. The .exe file will now be represented with your icon in Explorer.
*/
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "settings.h"
#include "record.h"
#include <QAudio>
#include <QMessageBox>
#include <QSettings>
#include <QLibrary>
#include <QUrl>
#include <QDesktopServices>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    connect(ui->actionSettings,SIGNAL(triggered()),this,SLOT(dosettings()));
    connect(ui->pushButtonRecord,SIGNAL(clicked()),this,SLOT(record()));
    connect(ui->actionAbout,SIGNAL(triggered()),this,SLOT(about()));
    connect (ui->actionHelp,SIGNAL(triggered()),this,SLOT(help()));
    readSettings();
    QLibrary mp3lame("libmp3lame",0);   // the "0" version number is needed on Linux, ignored on Windows
    mp3lame.load();
    if (!mp3lame.isLoaded()) {
        QMessageBox::warning(this,"Error","Failed to load libmp3lame library");
    }
    lameinit=(LAMEINIT) mp3lame.resolve("lame_init");
    lamesetinsamplerate=(LAMESETINSAMPLERATE) mp3lame.resolve("lame_set_in_samplerate");
    lamesetnumchannels=(LAMESETNUMCHANNELS) mp3lame.resolve("lame_set_num_channels");
    lamesetoutsamplerate=(LAMESETOUTSAMPLERATE) mp3lame.resolve("lame_set_out_samplerate");
    lamesetbwritevbrtag=(LAMESETBWRITEVBRTAG) mp3lame.resolve("lame_set_bWriteVbrTag");
    lamesetquality=(LAMESETQUALITY) mp3lame.resolve("lame_set_quality");
    lamesetmode=(LAMESETMODE) mp3lame.resolve("lame_set_mode");
    lamesetbrate=(LAMESETBRATE) mp3lame.resolve("lame_set_brate");
    lamegetbrate=(LAMEGETBRATE) mp3lame.resolve("lame_get_brate");
    lamesetcopyright=(LAMESETCOPYRIGHT) mp3lame.resolve("lame_set_copyright");
    lamesetoriginal=(LAMESETORIGINAL) mp3lame.resolve("lame_set_original");
    lameseterrorprotection=(LAMESETERRORPROTECTION) mp3lame.resolve("lame_set_error_protection");
    lamesetvbr=(LAMESETVBR) mp3lame.resolve("lame_set_VBR");
    lameinitparams=(LAMEINITPARAMS) mp3lame.resolve("lame_init_params");
    getlameversion=(GETLAMEVERSION) mp3lame.resolve("get_lame_version");
    getlameurl=(GETLAMEURL) mp3lame.resolve("get_lame_url");
    lameencodebuffer=(LAMEENCODEBUFFER) mp3lame.resolve("lame_encode_buffer");
    lameencodeflush=(LAMEENCODEFLUSH) mp3lame.resolve("lame_encode_flush");
    lameclose=(LAMECLOSE) mp3lame.resolve("lame_close");
    if (lameinit==NULL
        || lamesetinsamplerate==NULL
        || lamesetnumchannels==NULL
        || lamesetoutsamplerate==NULL
        || lamesetbwritevbrtag==NULL
        || lamesetquality==NULL
        || lamesetmode==NULL
        || lamesetbrate==NULL
        || lamegetbrate==NULL
        || lamesetcopyright==NULL
        || lamesetoriginal==NULL
        || lameseterrorprotection==NULL
        || lamesetvbr==NULL
        || lameinitparams==NULL
        || getlameversion==NULL
        || getlameurl==NULL
        || lameencodebuffer==NULL
        || lameencodeflush==NULL
        || lameclose==NULL) {
        QMessageBox::warning(this,"Error","Cannot initialize libmp3lame library");
    }
}

MainWindow::~MainWindow()
{
    delete ui;
}

// read setting from persistant storage
void MainWindow::readSettings()
{
    QSettings settings(QSettings::IniFormat,QSettings::UserScope,"krubow","sdrecorder"); // constructor, force INI file storage
    source=settings.value("source").toString();
    rate=settings.value("rate",44100).toInt();
    channels=settings.value("channels",2).toInt();
    soundactivated=settings.value("soundactivated").toBool();
    startlevel=settings.value("startlevel",5).toInt();
    starttime=settings.value("starttime",75).toInt(); //0.75 seconds
    stoplevel=settings.value("stoplevel",5).toInt();
    stoptime=settings.value("stoptime",2000).toInt(); // 20.00 seconds
    savefolder=settings.value("savefolder").toString();
    MP3Channels=settings.value("MP3Channels",0).toInt(); // default to mono
    MP3Frequency=settings.value("MP3Frequency",7).toInt(); // default to 44100
    MP3Bitrate=settings.value("MP3Bitrate",7).toInt(); // default to 64000
    MP3Encoding=settings.value("MP3Encoding",0).toInt(); // default to CBR
    MP3Quality=settings.value("MP3Quality",1).toInt(); // default to medium
}

void MainWindow::about(void)
{
    QMessageBox::about(this,"About SdRecorder",QString("<h1>SdRecorder 1.00</h1>by Keith Rubow<br>"
                            "<a href=\"http://www.krubow.com/\">www.krubow.com</a><br>"
                            "Using LAME MP3 encoder<br>LAME version %1<br>"
                            "<a href=\"%2\">%3")
                       .arg(getlameversion())
                       .arg(getlameurl()).arg(getlameurl()));
}

void MainWindow::help(void)
{
    QDesktopServices::openUrl(QUrl("file:///"+QCoreApplication::applicationDirPath()+"/SdRecorder.html"));
}

// process setting dialog box to change persistant settings
void MainWindow::dosettings()
{
    Settings dialog(this);

    dialog.setSource(source);
    dialog.setRate(rate);
    dialog.setChannels(channels);
    dialog.setSoundActivated(soundactivated);
    dialog.setStartLevel(startlevel);
    dialog.setStartTime(starttime);
    dialog.setStopLevel(stoplevel);
    dialog.setStopTime(stoptime);
    dialog.setSaveFolder(savefolder);
    dialog.setMP3Channels(MP3Channels);
    dialog.setMP3Frequency(MP3Frequency);
    dialog.setMP3Bitrate(MP3Bitrate);
    dialog.setMP3Encoding(MP3Encoding);
    dialog.setMP3Quality(MP3Quality);
    if (dialog.exec()) {
        source=dialog.getSource();
        rate=dialog.getRate();
        channels=dialog.getChannels();
        soundactivated=dialog.getSoundActivated();
        startlevel=dialog.getStartLevel();
        starttime=dialog.getStartTime();
        stoplevel=dialog.getStopLevel();
        stoptime=dialog.getStopTime();
        savefolder=dialog.getSaveFolder();
        MP3Channels=dialog.getMP3Channels();
        MP3Frequency=dialog.getMP3Frequency();
        MP3Bitrate=dialog.getMP3Bitrate();
        MP3Encoding=dialog.getMP3Encoding();
        MP3Quality=dialog.getMP3Quality();
        QSettings settings(QSettings::IniFormat,QSettings::UserScope,"krubow","sdrecorder"); // constructor, force INI file storage
        settings.setValue("source",source);
        settings.setValue("rate",rate);
        settings.setValue("channels",channels);
        settings.setValue("soundactivated",soundactivated);
        settings.setValue("startlevel",startlevel);
        settings.setValue("starttime",starttime);
        settings.setValue("stoplevel",stoplevel);
        settings.setValue("stoptime",stoptime);
        settings.setValue("savefolder",savefolder);
        settings.setValue("MP3Channels",MP3Channels);
        settings.setValue("MP3Frequency",MP3Frequency);
        settings.setValue("MP3Bitrate",MP3Bitrate);
        settings.setValue("MP3Encoding",MP3Encoding);
        settings.setValue("MP3Quality",MP3Quality);
    }
}

QString MainWindow::getFilename(void)
{
    return ui->lineEditFilename->text();
}

// open the record dialog to do the actual recording
void MainWindow::record(void)
{
    Record dialog(this);

    dialog.artist=ui->lineEditArtist->text();
    dialog.album=ui->lineEditAlbum->text();
    dialog.comment=ui->lineEditComment->text();
    dialog.year=ui->lineEditYear->text();
    dialog.exec();
}
